@@ -51,27 +51,35 @@ class _Error:
         self.offset = offset
         self.is_session_dead = False
         self.full_code = ""
-        if message is not None:
-            pos = message.find(":")
+        self._make_adjustments()
+
+    def _make_adjustments(self):
+        """
+        Make adjustments to the error, if needed, and calculate the full_code
+        attribute.
+        """
+        if self.message is not None:
+            pos = self.message.find(":")
             if pos > 0:
-                self.full_code = message[:pos]
-        if code != 0 or self.full_code.startswith("DPI-"):
+                self.full_code = self.message[:pos]
+        if self.code != 0 or self.full_code.startswith("DPI-"):
             args = {}
-            if code != 0:
-                driver_error_info = ERR_ORACLE_ERROR_XREF.get(code)
+            if self.code != 0:
+                driver_error_info = ERR_ORACLE_ERROR_XREF.get(self.code)
             else:
                 error_num = int(self.full_code[4:])
                 driver_error_info = ERR_DPI_ERROR_XREF.get(error_num)
             if driver_error_info is not None:
                 if isinstance(driver_error_info, tuple):
                     driver_error_num, pattern = driver_error_info
-                    args = re.search(pattern, message).groupdict()
+                    args = re.search(pattern, self.message).groupdict()
                 else:
                     driver_error_num = driver_error_info
                 if driver_error_num == ERR_CONNECTION_CLOSED:
                     self.is_session_dead = True
                 driver_error = _get_error_text(driver_error_num, **args)
                 self.message = f"{driver_error}\n{self.message}"
+                self.full_code = f"{ERR_PREFIX}-{driver_error_num:04}"
 
     def __str__(self):
         return self.message
@@ -152,6 +160,12 @@ ERR_INVALID_CONNECT_PARAMS = 2025
 ERR_INVALID_POOL_CLASS = 2026
 ERR_INVALID_POOL_PARAMS = 2027
 ERR_EXPECTING_LIST_FOR_ARRAY_VAR = 2028
+ERR_HTTPS_PROXY_REQUIRES_TCPS = 2029
+ERR_INVALID_LOB_OFFSET = 2030
+ERR_INVALID_ACCESS_TOKEN_PARAM = 2031
+ERR_INVALID_ACCESS_TOKEN_RETURNED = 2032
+ERR_EXPIRED_ACCESS_TOKEN = 2033
+ERR_ACCESS_TOKEN_REQUIRES_TCPS = 2034
 
 # error numbers that result in NotSupportedError
 ERR_TIME_NOT_SUPPORTED = 3000
@@ -165,14 +179,15 @@ ERR_DB_TYPE_NOT_SUPPORTED = 3007
 ERR_UNSUPPORTED_INBAND_NOTIFICATION = 3008
 ERR_SELF_BIND_NOT_SUPPORTED = 3009
 ERR_SERVER_VERSION_NOT_SUPPORTED = 3010
-ERR_SERVER_LOGON_TYPE_NOT_SUPPORTED = 3011
 ERR_NCHAR_CS_NOT_SUPPORTED = 3012
 ERR_UNSUPPORTED_PYTHON_TYPE_FOR_VAR = 3013
 ERR_LOB_OF_WRONG_TYPE = 3014
+ERR_UNSUPPORTED_VERIFIER_TYPE = 3015
+ERR_NO_CRYPTOGRAPHY_PACKAGE = 3016
 
 # error numbers that result in DatabaseError
 ERR_TNS_ENTRY_NOT_FOUND = 4000
-ERR_NO_PASSWORD = 4001
+ERR_NO_CREDENTIALS = 4001
 ERR_COLUMN_TRUNCATED = 4002
 ERR_ORACLE_NUMBER_NO_REPR = 4003
 ERR_INVALID_NUMBER = 4004
@@ -219,7 +234,7 @@ ERR_CONNECTION_FAILED = 6005
 ERR_ORACLE_ERROR_XREF = {
     28: ERR_CONNECTION_CLOSED,
     600: ERR_CONNECTION_CLOSED,
-    1005: ERR_NO_PASSWORD,
+    1005: ERR_NO_CREDENTIALS,
     24422: ERR_POOL_HAS_BUSY_CONNECTIONS,
     24349: ERR_ARRAY_DML_ROW_COUNTS_NOT_ENABLED,
     24459: ERR_POOL_NO_CONNECTION_AVAILABLE,
@@ -248,6 +263,8 @@ ERR_EXCEPTION_TYPES = {
 
 # error message formats
 ERR_MESSAGE_FORMATS = {
+    ERR_ACCESS_TOKEN_REQUIRES_TCPS:
+            'access_token requires use of the tcps protocol',
     ERR_ARGS_MUST_BE_LIST_OR_TUPLE:
             'arguments must be a list or tuple',
     ERR_ARGS_AND_KEYWORD_ARGS:
@@ -283,8 +300,12 @@ ERR_MESSAGE_FORMATS = {
     ERR_EXPECTING_VAR:
             'type handler should return None or the value returned by a call '
             'to cursor.var()',
+    ERR_EXPIRED_ACCESS_TOKEN:
+            'access token has expired',
     ERR_FEATURE_NOT_SUPPORTED:
             '{feature} is only supported in python-oracledb {driver_type} mode',
+    ERR_HTTPS_PROXY_REQUIRES_TCPS:
+            'https_proxy requires use of the tcps protocol',
     ERR_INCONSISTENT_DATATYPES:
             'cannot convert from data type {input_type} to {output_type}',
     ERR_INCORRECT_VAR_ARRAYSIZE:
@@ -295,6 +316,14 @@ ERR_MESSAGE_FORMATS = {
     ERR_INTEGER_TOO_LARGE:
             'internal error: read integer of length {length} when expecting '
             'integer of no more than length {max_length}',
+    ERR_INVALID_ACCESS_TOKEN_PARAM:
+            'invalid access token: value must be a string (for OAuth), a '
+            '2-tuple containing the token and private key strings (for IAM), '
+            'or a callable that returns a string or 2-tuple',
+    ERR_INVALID_ACCESS_TOKEN_RETURNED:
+            'invalid access token returned from callable: value must be a '
+            'string (for OAuth) or a 2-tuple containing the token and private '
+            'key strings (for IAM)',
     ERR_INVALID_BIND_NAME:
             'no bind placeholder named ":{name}" was found in the SQL text',
     ERR_INVALID_CONN_CLASS:
@@ -303,6 +332,8 @@ ERR_MESSAGE_FORMATS = {
             'invalid connect descriptor "{data}"',
     ERR_INVALID_CONNECT_PARAMS:
             'invalid connection params',
+    ERR_INVALID_LOB_OFFSET:
+            'LOB offset must be greater than zero',
     ERR_INVALID_MAKEDSN_ARG:
             '"{name}" argument contains invalid values',
     ERR_INVALID_NUMBER:
@@ -360,8 +391,11 @@ ERR_MESSAGE_FORMATS = {
             'python-oracledb in thin mode',
     ERR_NO_CONFIG_DIR:
             'no configuration directory to search for tnsnames.ora',
-    ERR_NO_PASSWORD:
-            'no password specified',
+    ERR_NO_CREDENTIALS:
+            'no credentials specified',
+    ERR_NO_CRYPTOGRAPHY_PACKAGE:
+            'python-oracledb thin mode cannot be used because the '
+            'cryptography package is not installed',
     ERR_NO_STATEMENT:
             'no statement specified and no prior statement prepared',
     ERR_NO_STATEMENT_EXECUTED:
@@ -400,8 +434,6 @@ ERR_MESSAGE_FORMATS = {
             'binding to self is not supported',
     ERR_CONNECTION_CLOSED:
             'the database or network closed the connection',
-    ERR_SERVER_LOGON_TYPE_NOT_SUPPORTED:
-            'server logon version is not supported',
     ERR_SERVER_VERSION_NOT_SUPPORTED:
             'connections to this database server version are not supported '
             'by python-oracledb in thin mode',
@@ -431,6 +463,9 @@ ERR_MESSAGE_FORMATS = {
             '{db_type_name}',
     ERR_UNSUPPORTED_VAR_SET:
             'variable of type {db_type_name} does not support being set',
+    ERR_UNSUPPORTED_VERIFIER_TYPE:
+            'password verifier type 0x{verifier_type:x} is not supported by '
+            'python-oracledb in thin mode',
     ERR_WALLET_FILE_MISSING:
             'cannot connect to database. Wallet file {name} was not found',
     ERR_WRONG_ARRAY_DEFINITION:
