@@ -69,7 +69,7 @@ extern "C" {
 
 // define ODPI-C version information
 #define DPI_MAJOR_VERSION   4
-#define DPI_MINOR_VERSION   4
+#define DPI_MINOR_VERSION   5
 #define DPI_PATCH_LEVEL     0
 #define DPI_VERSION_SUFFIX
 
@@ -1005,6 +1005,13 @@ DPI_EXPORT int dpiConn_newDeqOptions(dpiConn *conn, dpiDeqOptions **options);
 // create a new enqueue options object and return it
 DPI_EXPORT int dpiConn_newEnqOptions(dpiConn *conn, dpiEnqOptions **options);
 
+// create a new, empty JSON
+DPI_EXPORT int dpiConn_newJson(dpiConn *conn, dpiJson **json);
+
+// create a new AQ queue with JSON payload
+DPI_EXPORT int dpiConn_newJsonQueue(dpiConn *conn, const char *name,
+        uint32_t nameLength, dpiQueue **queue);
+
 // create a new message properties object and return it
 DPI_EXPORT int dpiConn_newMsgProps(dpiConn *conn, dpiMsgProps **props);
 
@@ -1470,6 +1477,9 @@ DPI_EXPORT int dpiMsgProps_getOriginalMsgId(dpiMsgProps *props,
 DPI_EXPORT int dpiMsgProps_getPayload(dpiMsgProps *props, dpiObject **obj,
         const char **value, uint32_t *valueLength);
 
+// return the payload of the message (JSON)
+DPI_EXPORT int dpiMsgProps_getPayloadJson(dpiMsgProps *props, dpiJson **json);
+
 // return the priority of the message
 DPI_EXPORT int dpiMsgProps_getPriority(dpiMsgProps *props, int32_t *value);
 
@@ -1502,6 +1512,9 @@ DPI_EXPORT int dpiMsgProps_setOriginalMsgId(dpiMsgProps *props,
 DPI_EXPORT int dpiMsgProps_setPayloadBytes(dpiMsgProps *props,
         const char *value, uint32_t valueLength);
 
+// set the payload of the message (as JSON)
+DPI_EXPORT int dpiMsgProps_setPayloadJson(dpiMsgProps *props, dpiJson *json);
+
 // set the payload of the message (as an object)
 DPI_EXPORT int dpiMsgProps_setPayloadObject(dpiMsgProps *props,
         dpiObject *obj);
