@@ -739,6 +739,7 @@ class TestCase(test_env.BaseTestCase):
             admin_cursor.execute(sql)
         self.assertRaisesRegex(oracledb.DatabaseError, "^DPY-4011:",
                                cursor.execute, "select user from dual")
+        self.assertFalse(conn.is_healthy())
 
     def test_4359_kill_conn_in_context_manager(self):
         "4359 - kill connection in cursor context manager"
@@ -784,5 +785,36 @@ class TestCase(test_env.BaseTestCase):
             self.assertEqual(cursor.rowcount, 1)
         self.assertEqual(cursor.rowcount, -1)
 
+    def test_4362_change_of_bind_type_with_define(self):
+        "4362 - changing bind type with define needed"
+        self.cursor.execute("truncate table TestClobs")
+        row_for_1 = (1, "Short value 1")
+        row_for_56 = (56, "Short value 56")
+        for data in (row_for_1, row_for_56):
+            self.cursor.execute("""
+                    insert into TestClobs (IntCol, ClobCol)
+                    values (:1, :2)""", data)
+        sql = "select IntCol, ClobCol from TestClobs where IntCol = :int_col"
+        with test_env.FetchLobsContextManager(False):
+            self.cursor.execute(sql, int_col="1")
+            self.assertEqual(self.cursor.fetchone(), row_for_1)
+            self.cursor.execute(sql, int_col="56")
+            self.assertEqual(self.cursor.fetchone(), row_for_56)
+            self.cursor.execute(sql, int_col=1)
+            self.assertEqual(self.cursor.fetchone(), row_for_1)
+
+    def test_4363_multiple_parse(self):
+        "4363 - test calling cursor.parse() twice with the same statement"
+        self.cursor.execute("truncate table TestTempTable")
+        data = (4363, "Value for test 4363")
+        self.cursor.execute("""
+                insert into TestTempTable (IntCol, StringCol1)
+                values (:1, :2)""", data)
+        sql = "update TestTempTable set StringCol1 = :v where IntCol = :i"
+        for i in range(2):
+            self.cursor.parse(sql)
+            self.cursor.execute(sql, ("Updated value", data[0]))
+
+
 if __name__ == "__main__":
     test_env.run_test_cases()
