@@ -40,6 +40,7 @@
 #   PYO_TEST_WALLET_LOCATION: location of wallet file (thin mode, mTLS)
 #   PYO_TEST_WALLET_PASSWORD: password for wallet file (thin mode, mTLS)
 #   PYO_TEST_DRIVER_MODE: python-oracledb mode (thick or thin) to use
+#   PYO_TEST_EXTERNAL_USER: user for testing external authentication
 #
 # PYO_TEST_CONNECT_STRING can be set to an Easy Connect string, or a
 # Net Service Name from a tnsnames.ora file or external naming service,
@@ -212,6 +213,10 @@ def get_wallet_password():
     if get_is_thin():
         return get_value("WALLET_PASSWORD", "Wallet Password", password=True)
 
+def get_external_user():
+    if not get_is_thin():
+        return get_value("EXTERNAL_USER", "External User")
+
 def run_sql_script(conn, script_name, **kwargs):
     statement_parts = []
     cursor = conn.cursor()
@@ -321,6 +326,23 @@ class BaseTestCase(unittest.TestCase):
     def assertRoundTrips(self, n):
         self.assertEqual(self.round_trip_info.get_value(), n)
 
+    def get_and_clear_queue(self, queue_name, payload_type=None,
+                            message="not supported with this client/server " \
+                                    "combination"):
+        if payload_type == "JSON":
+            if get_client_version() < (21, 0) \
+                    or get_server_version() < (21, 0):
+                self.skipTest(message)
+        elif isinstance(payload_type, str):
+            payload_type = self.connection.gettype(payload_type)
+        queue = self.connection.queue(queue_name, payload_type)
+        queue.deqoptions.wait = oracledb.DEQ_NO_WAIT
+        queue.deqoptions.deliverymode = oracledb.MSG_PERSISTENT_OR_BUFFERED
+        queue.deqoptions.visibility = oracledb.DEQ_IMMEDIATE
+        while queue.deqone():
+            pass
+        return self.connection.queue(queue_name, payload_type)
+
     def get_db_object_as_plain_object(self, obj):
         if obj.type.iscollection:
             element_values = []
